# direct volume rendering of a CT of a mummy
#
# Konstantin Shkurko
# 11.02.12

import vtk

whichToProcess = 2
if( whichToProcess==0 ):
	fileToOpen = 'mummy.50.vtk'
elif( whichToProcess==1 ):
	fileToOpen = 'mummy.80.vtk'
else:
	fileToOpen = 'mummy.128.vtk'
#rayCastStepSize = 0.1
#rayCastStepSize = 0.3
#rayCastStepSize = 1
rayCastStepSize = 0.2

doFinalStuff = 0
doNearestNeighboor = 0

# Create the reader for the data
reader = vtk.vtkStructuredPointsReader()
reader.SetFileName( fileToOpen )

# Create transfer mapping scalar value to opacity
opacityTransferFunction = vtk.vtkPiecewiseFunction()
opacityTransferFunction.AddPoint( 74, 0.0 )
opacityTransferFunction.AddPoint( 78, 0.35 )
opacityTransferFunction.AddPoint( 84, 0.55 )
opacityTransferFunction.AddPoint( 90, 0.0 )
opacityTransferFunction.AddPoint( 120, 0.0 )
opacityTransferFunction.AddPoint( 140, 1.0 )

# Create transfer mapping scalar value to color
colorTransferFunction = vtk.vtkColorTransferFunction()
colorTransferFunction.AddRGBPoint( 0.0, 0.0, 0.0, 0.0 )
colorTransferFunction.AddRGBPoint( 80, 1.0, 0.4, 0.3 )
colorTransferFunction.AddRGBPoint( 85, 0, 0, 0 )
colorTransferFunction.AddRGBPoint( 90, 1, 1, 1 )
colorTransferFunction.AddRGBPoint( 255.0, 1, 1, 1 )

volumeGradientOpacity = vtk.vtkPiecewiseFunction()
volumeGradientOpacity.AddPoint( 0, 0.0 )
volumeGradientOpacity.AddPoint( 5, 0 )
volumeGradientOpacity.AddPoint( 7, 1 )

# The property describes how the data will look
volumeProperty = vtk.vtkVolumeProperty()
volumeProperty.SetColor( colorTransferFunction )
volumeProperty.SetScalarOpacity( opacityTransferFunction )
volumeProperty.SetGradientOpacity( volumeGradientOpacity )
volumeProperty.ShadeOn()
volumeProperty.SetSpecular( 0.2 )
if( doNearestNeighboor==1 ):
	volumeProperty.SetInterpolationTypeToNearest()
else:
	volumeProperty.SetInterpolationTypeToLinear()

# The mapper / ray cast function know how to render the data
compositeFunction = vtk.vtkVolumeRayCastCompositeFunction()
volumeMapper = vtk.vtkVolumeRayCastMapper()
volumeMapper.SetVolumeRayCastFunction( compositeFunction )
volumeMapper.SetInputConnection( reader.GetOutputPort() )
volumeMapper.SetSampleDistance( rayCastStepSize )

# The volume holds the mapper and the property and
# can be used to position/orient the volume
volume = vtk.vtkVolume()
volume.SetMapper( volumeMapper )
volume.SetProperty( volumeProperty )

# Create the standard renderer, render window and interactor
ren    = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer( ren )
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow( renWin )

istyle = vtk.vtkInteractorStyleSwitch()
iren.SetInteractorStyle( istyle )
istyle.SetCurrentStyleToTrackballCamera()

if( doFinalStuff==1 ):
	volume.RotateZ( -30 )

ren.AddVolume( volume )
ren.SetBackground( 0.8, 0.8, 0.8 )
renWin.SetSize( 600, 600 )
ren.ResetCamera()
if( doFinalStuff==1 ):
	ren.GetActiveCamera().Zoom( 2.1 )
else:
	ren.GetActiveCamera().Zoom( 1.5 )
ren.GetActiveCamera().Elevation( -90 )
renWin.Render()

def CheckAbort( obj, event ):
	if obj.GetEventPending() != 0:
		obj.SetAbortRender( 1 )

renWin.AddObserver( "AbortCheckEvent", CheckAbort )

iren.Initialize()
renWin.Render()
iren.Start()
