# isosurface rendering of a CT of a mummy
#
# Konstantin Shkurko
# 11.02.12

import vtk

whichToProcess = 2
if( whichToProcess==0 ):
	fileToOpen = 'mummy.50.vtk'
elif( whichToProcess==1 ):
	fileToOpen = 'mummy.80.vtk'
else:
	fileToOpen = 'mummy.128.vtk'


# Create the reader for the data
reader = vtk.vtkStructuredPointsReader()
reader.SetFileName( fileToOpen )

# Marching Cubes - skin
isoSkull = vtk.vtkContourFilter()
isoSkull.SetInputConnection( reader.GetOutputPort() )
isoSkull.ComputeScalarsOn()
isoSkull.SetValue( 0, 80 )
isoSkull.Update()

isoSkullMapper = vtk.vtkPolyDataMapper()
isoSkullMapper.SetInputConnection( isoSkull.GetOutputPort() )
isoSkullMapper.ScalarVisibilityOff()

actorSkull = vtk.vtkActor()
actorSkull.SetMapper( isoSkullMapper )
actorSkull.GetProperty().SetColor( 1.0, 0.4, 0.3 )
actorSkull.GetProperty().SetOpacity( 0.3 )

# Marching Cubes - skull
isoSkin = vtk.vtkContourFilter()
isoSkin.SetInputConnection( reader.GetOutputPort() )
isoSkin.ComputeScalarsOn()
isoSkin.SetValue( 0, 140 )
isoSkin.Update()

isoSkinMapper = vtk.vtkPolyDataMapper()
isoSkinMapper.SetInputConnection( isoSkin.GetOutputPort() )
isoSkinMapper.ScalarVisibilityOff()

actorSkin = vtk.vtkActor()
actorSkin.SetMapper( isoSkinMapper )
actorSkin.GetProperty().SetColor( 1.0, 1.0, 1.0 )
actorSkin.GetProperty().SetOpacity( 1 )

# Create the standard renderer, render window and interactor
ren    = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer( ren )
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow( renWin )

istyle = vtk.vtkInteractorStyleSwitch()
iren.SetInteractorStyle( istyle )
istyle.SetCurrentStyleToTrackballCamera()

ren.AddActor( actorSkull )
ren.AddActor( actorSkin )
ren.SetBackground( 0.8, 0.8, 0.8 )
renWin.SetSize( 600, 600 )
ren.ResetCamera()
ren.GetActiveCamera().Zoom( 1.5 )
ren.GetActiveCamera().Elevation( -90 )
renWin.Render()

def CheckAbort( obj, event ):
	if obj.GetEventPending() != 0:
		obj.SetAbortRender( 1 )

renWin.AddObserver( "AbortCheckEvent", CheckAbort )

iren.Initialize()
renWin.Render()
iren.Start()