# MIP rendering of a CT of a mummy
#
# Konstantin Shkurko
# 11.02.12

import vtk

whichToProcess = 2
if( whichToProcess==0 ):
	fileToOpen = 'mummy.50.vtk'
elif( whichToProcess==1 ):
	fileToOpen = 'mummy.80.vtk'
else:
	fileToOpen = 'mummy.128.vtk'
#rayCastStepSize = 0.1
rayCastStepSize = 0.3
#rayCastStepSize = 1

doMIP = 1

# Create the reader for the data
reader = vtk.vtkStructuredPointsReader()
reader.SetFileName( fileToOpen )

# Create transfer mapping scalar value to opacity
opacityTransferFunction = vtk.vtkPiecewiseFunction()
if( doMIP==1 ):
	opacityTransferFunction.AddPoint( 70, 0.0 )
	opacityTransferFunction.AddPoint( 90, 0.7 )
	opacityTransferFunction.AddPoint( 140, 1 )
	opacityTransferFunction.AddPoint( 255, 1 )
else:
	opacityTransferFunction.AddPoint( 70, 0.0 )
	opacityTransferFunction.AddPoint( 90, 0.3 )
	opacityTransferFunction.AddPoint( 140, 0.6 )
	opacityTransferFunction.AddPoint( 255, 0.6 )


# Create transfer mapping scalar value to color
colorTransferFunction = vtk.vtkColorTransferFunction()
colorTransferFunction.AddRGBPoint( 0.0, 0.0, 0.0, 0.0 )
colorTransferFunction.AddRGBPoint( 10, 1, 1, 1 )
colorTransferFunction.AddRGBPoint( 90, 1, 1, 1 )
colorTransferFunction.AddRGBPoint( 140, 0.5, 0.5, 0.5 )
colorTransferFunction.AddRGBPoint( 255.0, 0.2, 0.2, 0.2 )

# The property describes how the data will look
volumeProperty = vtk.vtkVolumeProperty()
volumeProperty.SetColor( colorTransferFunction )
volumeProperty.SetScalarOpacity( opacityTransferFunction )
volumeProperty.SetInterpolationTypeToLinear()

# The mapper / ray cast function know how to render the data
if( doMIP==1 ):
	compositeFunction = vtk.vtkVolumeRayCastMIPFunction()
else:
	compositeFunction = vtk.vtkVolumeRayCastCompositeFunction()
volumeMapper = vtk.vtkVolumeRayCastMapper()
volumeMapper.SetVolumeRayCastFunction( compositeFunction )
volumeMapper.SetInputConnection( reader.GetOutputPort() )
volumeMapper.SetSampleDistance( rayCastStepSize )

# The volume holds the mapper and the property and
# can be used to position/orient the volume
volume = vtk.vtkVolume()
volume.SetMapper( volumeMapper )
volume.SetProperty( volumeProperty )

# Create the standard renderer, render window and interactor
ren    = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer( ren )
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow( renWin )

istyle = vtk.vtkInteractorStyleSwitch()
iren.SetInteractorStyle( istyle )
istyle.SetCurrentStyleToTrackballCamera()

ren.AddVolume( volume )
ren.SetBackground( 0, 0, 0 )
renWin.SetSize( 600, 600 )
ren.ResetCamera()
ren.GetActiveCamera().Zoom( 1.5 )
ren.GetActiveCamera().Elevation( -90 )
renWin.Render()

def CheckAbort( obj, event ):
	if obj.GetEventPending() != 0:
		obj.SetAbortRender( 1 )

renWin.AddObserver( "AbortCheckEvent", CheckAbort )

iren.Initialize()
renWin.Render()
iren.Start()
